package test;


import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.arquillian.test.api.ArquillianResource;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.asset.EmptyAsset;
import org.jboss.shrinkwrap.api.asset.StringAsset;
import org.jboss.shrinkwrap.api.container.ManifestContainer;
import org.jboss.shrinkwrap.api.spec.EnterpriseArchive;
import org.jboss.shrinkwrap.api.spec.JavaArchive;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.junit.Test;
import org.junit.runner.RunWith;
import java.io.IOException;
import java.net.URL;

import static org.junit.Assert.*;


@RunWith(Arquillian.class)
public class SessionBeanTest {
    @Deployment(testable = false)
    public static Archive<?> createDeployment() {
        return ShrinkWrap.create(EnterpriseArchive.class)
                .addAsLibrary(
                        ShrinkWrap.create(JavaArchive.class, "lib.jar")
                                .addClasses(RequestBean.class)
                                .addClasses(EventLog.class)
                                .addAsManifestResource(EmptyAsset.INSTANCE, "beans.xml")
                )
                .addAsModule(
                        ShrinkWrap.create(WebArchive.class, "test.war")
                                .addClasses(
                                        EventLogServlet.class,
                                        LogoutServlet.class,
                                        SessionBeanServlet.class,
                                        SessionBeanProducer.class
                                )
                                .addAsManifestResource(new StringAsset("Class-Path: lib/lib.jar"), ManifestContainer.DEFAULT_MANIFEST_NAME)
                                .addAsWebInfResource(EmptyAsset.INSTANCE, "beans.xml")
                )
                .addAsApplicationResource(
                        SessionBeanTest.class.getResource("application.xml"), "application.xml"
                );
    }

    @Test
    public void destroysSessionScopedBean() throws IOException {
        String[] uuidAndSession = ResponseUtil.readLines(new URL(baseUrl, "bean"));
        ResponseUtil.readFully(new URL(baseUrl, "logout;jsessionid=" + uuidAndSession[1]));
        String[] events = ResponseUtil.readLines(new URL(baseUrl, "events"));

        assertArrayEquals(
                new String[]{
                        "created " + uuidAndSession[0],
                        "calling request bean",
                        "beanMethod called",
                        "request bean called"
                },
                events
        );
    }

    @ArquillianResource
    private URL baseUrl;
}
